//QSuperList 1.1 implementation. See license.txt for licensing
//copyright 2000, Spanware, Inc.
//by David Koretzky


#include "qsuperlist.h"

#include <qwidget.h>
#include <qmessagebox.h>
#include <qpixmap.h>

#include "qinputbox.h"

#include <stdio.h>

#include "valadd.xpm"
#include "delvalue.xpm"

QSuperList :: QSuperList(QWidget* parent=0, const char * name=0, WFlags f=0) : QFrame(parent, name, f)
{
	
	listBox = new QListBox(this, "listbox", 0);
	connect (listBox, SIGNAL(selected(int)), SLOT(ListBoxDoubleClicked(int)) );

	
	btnAdd = new QPushButton("", this);
	QPixmap p(valadd);
	btnAdd -> setPixmap(p);
	connect (btnAdd, SIGNAL( clicked() ), SLOT( AddClicked ()) );		
	
	btnDelete = new QPushButton("", this);
	QPixmap p2(delvalue);
	btnDelete -> setPixmap(p2);
	connect (btnDelete, SIGNAL( clicked() ), SLOT( DeleteClicked()) );
	
	m_nMaxValues = 0;
	m_nMaxLength = 0;
	m_nButtonSize= 30;
	m_bAllowBlanks = false;
	
	EnableButtons();
	
	alignButtons(baTop);	
	setFrameStyle( QFrame::Panel | QFrame::Sunken);	
}


QSuperList :: ~QSuperList()
{
}


void QSuperList :: addItem(QString str)
{
	if (m_nMaxValues == 0 || m_nMaxValues > listBox -> count() && CheckLength( str) )
		listBox -> insertItem(str);
}


void QSuperList :: DeleteClicked()
{
	if (listBox -> currentItem() > - 1)
		listBox -> removeItem( listBox -> currentItem() );
	EnableButtons();
}


void QSuperList :: setMaxValues(int maxValues)
{
	m_nMaxValues = maxValues;
	EnableButtons();
}


void QSuperList :: setMaxLength(int maxLength)
{
	m_nMaxLength = maxLength;
}

int QSuperList :: getMaxValues() const
{
	return m_nMaxValues;
}

int QSuperList :: getMaxLength() const
{
	return m_nMaxLength;
}


void QSuperList :: ListBoxDoubleClicked(int index)
{
	QString result = listBox -> text(index);
	QInputBox b("Edit Value", "Edit Value", result);
	if ( CheckLength( result ) )
		listBox -> changeItem( result, listBox -> currentItem());
}


void QSuperList :: AddClicked()
{
	QString result;
	QInputBox b("Add Value", "Enter value to add", result);
	 if (b.result() ==  b.Accepted) //wanna know why I need this? email for an explanation
	  if (CheckLength(result))
		addItem(result);
	EnableButtons();
}


bool QSuperList :: CheckLength(QString str)
{
	if (str.isEmpty() && !m_bAllowBlanks)
		return false;
		
	if (str.length() <= m_nMaxLength || m_nMaxLength == 0)
		return true;
	else
		return false;
}	

QListBox* QSuperList :: getListBox()
{
	return listBox;
}

void QSuperList :: EnableButtons()
{
	btnAdd -> setEnabled ( listBox -> count() < m_nMaxValues || m_nMaxValues == 0);
	btnDelete -> setEnabled (listBox -> count() > 0);
}


void QSuperList :: alignButtons(QDirection d)
{
	m_buttonAlignment = d;
	switch (d)
	{
		case (baTop):
		{
			listBox -> setGeometry ( lineWidth(), m_nButtonSize, width() , height() - m_nButtonSize);		
			btnAdd -> setGeometry ( lineWidth(), lineWidth(), m_nButtonSize, m_nButtonSize);			
			btnDelete -> setGeometry( m_nButtonSize + 1, lineWidth(), m_nButtonSize, m_nButtonSize);			
			break;
		}
		case (baBottom):
		{
			listBox -> setGeometry ( lineWidth(), lineWidth(), width() , height() - m_nButtonSize);		
			btnAdd -> setGeometry ( lineWidth(), height() - m_nButtonSize, m_nButtonSize, m_nButtonSize);			
			btnDelete -> setGeometry( m_nButtonSize + 1, height() - m_nButtonSize, m_nButtonSize, m_nButtonSize);			
			break;
		}
		case (baLeft):
		{
			listBox -> setGeometry ( m_nButtonSize , lineWidth(), width() , height());		
			btnAdd -> setGeometry ( lineWidth(), lineWidth(), m_nButtonSize, m_nButtonSize);			
			btnDelete -> setGeometry( lineWidth(), m_nButtonSize + 1, m_nButtonSize, m_nButtonSize);			
  		break;
		}
		case (baRight):
		{
			listBox -> setGeometry ( lineWidth() , lineWidth(), width() - m_nButtonSize, height());		
			btnAdd -> setGeometry ( width() - m_nButtonSize, lineWidth(), m_nButtonSize, m_nButtonSize);			
			btnDelete -> setGeometry( width()- m_nButtonSize, m_nButtonSize + 1, m_nButtonSize, m_nButtonSize);			
			break;
		}
		default:
			alignButtons(baTop);
	}
}

void QSuperList :: setButtonSize(unsigned int size)
{
	m_nButtonSize = size;
	alignButtons ( m_buttonAlignment );
}

unsigned int QSuperList :: getButtonSize() const
{
	return m_nButtonSize;
}

void QSuperList :: setAllowBlanks(bool b)
{
	m_bAllowBlanks = b;
}

bool QSuperList :: getAllowBlanks() const
{
	return m_bAllowBlanks;
}

void QSuperList :: clear()
{
	listBox -> clear();
}


void QSuperList :: show()
{
	//added 6/13 - sometimes the listbox does not show
	//up right after construction.  Even though I call
	//alignButtons from the constructor, in certain dialog
	//boxes the listbox is mashed.  One way around that is to
	//have the client call alignButtons after construction, but
	//lets face it, that is a pain in the ass, so I override show
	//here, make sure the ancestor is fully setup and then call
	//alignButtons.  I am investigating this, as when it is used on
	//a mainWidget it works fine.
	QFrame::show();
	alignButtons(m_buttonAlignment);	
}
